package com.ruoyi.system.service;

import java.util.List;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.system.domain.InfoUser;
import com.ruoyi.system.domain.PosReview;

/**
 * reviewService接口
 * 
 * @author ruoyi
 * @date 2023-06-08
 */
public interface IPosReviewService extends IService<PosReview>
{
    /**
     * 查询review
     * 
     * @param id review主键
     * @return review
     */
    public PosReview selectPosReviewById(Long id);

    /**
     * 查询review列表
     * 
     * @param posReview review
     * @return review集合
     */
    public List<PosReview> selectPosReviewList(PosReview posReview);

    /**
     * 新增review
     * 
     * @param posReview review
     * @return 结果
     */
    public int insertPosReview(PosReview posReview);

    /**
     * 修改review
     * 
     * @param posReview review
     * @return 结果
     */
    public int updatePosReview(PosReview posReview);

    /**
     * 批量删除review
     * 
     * @param ids 需要删除的review主键集合
     * @return 结果
     */
    public int deletePosReviewByIds(Long[] ids);

    /**
     * 删除review信息
     * 
     * @param id review主键
     * @return 结果
     */
    public int deletePosReviewById(Long id);
}
